package ch.hslu.swk.g03.logger.component;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Date;

import ch.hslu.loggerinterface.Logger;
import ch.hslu.loggerinterface.LoggerSetup;
import ch.hslu.loggerinterface.Loglevel;
import ch.hslu.swk.g03.logger.common.ConverterUtils;
import ch.hslu.swk.g03.logger.common.LogMessage;



public class Loggi implements Logger {

	private LoggiSetup setup = new LoggiSetup();
	private boolean connected = false;
	private Socket clientSocket;
	private DataOutputStream outStream;
	private BufferedReader inStream;
	
	
	/**
	 * Returns the current setup
	 */
	@Override
	public LoggerSetup getSetup() {

		return setup;
	}

	
	/**
     * Sends log messages to a server.
     * Logs a message with defined source and log level.
     *
     * @param level Sets the severity of the log message @see ch.hslu.swk.f1301.loggerinterface.Loglevel
     * @param message Sets the message to log. Maximum of 10000 characters
     */
	@Override
	public synchronized void log(Loglevel level, String message) {

		if (level.getLoglevel() > setup.getLoglevel().getLoglevel()) return;
		
		connect();
		
		if (message.length() > 10000) message = message.substring(0, 10000);
		
		
		
		// Create a new log message and set its properties
		LogMessage logMessage = new LogMessage();
		
		logMessage.setLoglevel(level);
		logMessage.setMessage(message);
		logMessage.setSource(setup.getSource());
		logMessage.setTimestamp(new Date());
		
		try {
			
			logMessage.setComputer(InetAddress.getLocalHost().getHostName());			
		
			// Connect the client if necessary
			if (!connected) connect();
			
			byte[] data = ConverterUtils.serialize(logMessage);
			
			// Send the length of the data
			outStream.writeInt(data.length);
					
			// Convert the object and send the serialize data to the server
			outStream.write(data);
			
			// Send end of data
			outStream.writeInt(-1);
		} catch (Exception ex) {

			// Print stack trace
			ex.printStackTrace();
		}
		
		disconnect();
	}

	/**
     * sets the setup
     * 
     * @param setup the setup to set
     */
	@Override
	public void setSetup(LoggerSetup arg0) {
		
		// Copy properties
		setup.setPort(arg0.getPort());
		setup.setServer(arg0.getServer());
		setup.setSource(arg0.getSource());
		setup.setLoglevel(arg0.getLoglevel());
		
		// Connect to the server if possible
		if (!setup.getServer().equals("")) {			
				
			// Connection test
			log(Loglevel.OFF, "Connection Test");
		}
	}
	
	
	/**
	 * Connects to a tcp server, which is defined in the setup
	 */
	private void connect(){
		
		try {
			if (clientSocket == null) {
				
				// Close connection
				if (connected) {
					disconnect();
				}			
			}
			
			// Try to connect
			clientSocket = new Socket(setup.getServer(), setup.getPort());
			  
			outStream = new DataOutputStream(clientSocket.getOutputStream());
			inStream = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
			
			connected = true;
		} catch (Exception ex) {
			
			// Print stack trace
			ex.printStackTrace();
		}
	}

	/**
	 * Disconnects the current connection to the logger server
	 */
	private void disconnect() {
		
		if(clientSocket != null) {

			try {
				// Close stream and the tcp socket
				outStream.close();
				inStream.close();
				clientSocket.close();
			} catch (Exception ex) {
				
				// Print stack trace
				ex.printStackTrace();
			}
						
			outStream = null;
			inStream = null;
			clientSocket = null;
		}
		
		connected = false;
	}
}
