package ch.hslu.swk.g03.logger.server;

import java.io.IOException;
import java.rmi.AlreadyBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.Properties;

/**
 * Logger Server Main.
 *
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public class ServerMain {

    private static final int TCP_START_CYCLE = 100;
    private static final int LOOP_CYCLE = 30000; // 10sec
    private static boolean listening = true;

    /**
     * Server Main Method.
     *
     * @param args
     *            cmd params
     */
    public static void main(final String[] args) {
	// init new LoggerServer
	final LoggerServer myServer = new LoggerServer();

	// Load Settings from properties file
	if (!myServer.loadSettings()) {
	    System.err.println("[ERROR:Main] LoadSettings failed...");
	    System.exit(1); // force quit
	}

	// init LoggerDB, Logfile
	LoggerDB.getInstance().setLogfileName(myServer.getLogFileName());
	try {
	    LoggerDB.getInstance().openLogfile();
	} catch (IOException e1) {
	    System.err.println("[ERROR:Main] LoggerDB openLogfile() failed...");
	    System.exit(1); // force quit
	}

	// init TCPServer
	final ServerTCP myServerTCP = ServerTCP.getInstance();
	myServerTCP.setPort(myServer.getTcpPort());
	myServerTCP.start();

	while (myServerTCP.isServerStarting()) {
	    System.out.println("[INFO:Main] waiting till ServerTCP is ready...");
	    try {
		Thread.sleep(TCP_START_CYCLE);
	    } catch (InterruptedException e) {
		System.err.println("[WARNING:Main] sleepinterrupt");
	    }
	}

	if (!myServerTCP.isServerListening()) {
	    System.err.println("[ERROR:Main] couldn't start ServerTCP!");
	    System.exit(1); // force quit
	}

	// init Corba Server
	final Properties myProb = new Properties();
	try {
	    // load a properties file from class path, inside static method
	    myProb.load(ServerMain.class.getClassLoader().getResourceAsStream("server.properties"));
	} catch (Exception ex) {
	    // load properties-file failed
	    System.err.println("[ERROR:Main] couldn't load Corba-Settings...");
	    return;
	}
	System.out.println("Corba Nameservice: " + myProb.getProperty("corba.nameservice"));
	final ServerCorba myServerCorba = new ServerCorba(myProb);
	

	// init RMI Server
	try {
	    // Server an der Registry anmelden
	    /*
	    LocateRegistry.createRegistry(Registry.REGISTRY_PORT);
	    Naming.rebind(myServer.getRmiServerName(), new ServerRMI());
	    */
	    
	    // Start the registry
            Registry registry = LocateRegistry.createRegistry(myServer.getRmiPort());
            // run the server
            registry.bind(myServer.getRmiServerName(), new ServerRMI());
	    
	    System.out.println("[INFO:ServerRMI] Server '" + myServer.getRmiServerName() + "', Port '" + myServer.getRmiPort()
		    + "' is online and ready...");
	} catch (RemoteException ex) {
	    System.err.println("[ERROR:ServerRMI RemoteException] " + ex.getMessage());
	    System.exit(1); // force quit
	} catch (AlreadyBoundException ex) {
	    System.err.println("[ERROR:ServerRMI AlreadyBoundException] " + ex.getMessage());
	    System.exit(1); // force quit
	}

	// endless-loop !!!
	while (listening) {
	    // display connected Clients
	    // myServerTCP.printConnectedClients();

	    // display received messages since start
	    //LoggerDB.getInstance().printAllMessages();

	    // Sleep
	    try {
		Thread.sleep(LOOP_CYCLE);
	    } catch (InterruptedException e) {
		System.err.println("[WARNING:Main] sleepinterrupt");
	    }
	}

	System.out.println("[INFO:Main] stop myServerTCP...");
	myServerTCP.stopServer();

	try {
	    LoggerDB.getInstance().closeLogfile();
	} catch (IOException e1) {
	    System.err.println("[ERROR:Main] LoggerDB closeLogfile() failed...");
	}
	System.out.println("[INFO:Main] bye bye...");
	System.exit(0);
    }
}
