package ch.hslu.swk.g03.logger.server;

import java.io.IOException;
import java.net.ServerSocket;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * LoggerServer TCP-Server.
 *
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public final class ServerTCP extends Thread {
    private static ServerTCP instance;
    private static int tcpPort;
    private boolean starting = true;
    private boolean listening;
    private final ArrayList<ServerTCPConnThread> clientList;

    /**
     * Default Constructor.
     */
    private ServerTCP() {
	super("ServerTCPThread");
	this.clientList = new ArrayList<ServerTCPConnThread>();
    }

    /**
     * Returns new ServerTCP - Singleton.
     *
     * @return ServerTCP instance
     */
    public static ServerTCP getInstance() {
	if (ServerTCP.instance == null) {
	    try {
		ServerTCP.instance = new ServerTCP();
	    } catch (Exception e) {
		System.err.println("[ERROR:ServerTCP] getInstance()");
		System.err.println(e.getStackTrace());
	    }
	}
	return ServerTCP.instance;
    }

    /**
     * Thread-Start Method.
     */
    public void run() {
	ServerSocket serverSocket = null;
	starting = true;

	try {
	    serverSocket = new ServerSocket(ServerTCP.tcpPort);
	    listening = true;
	} catch (IOException e) {
	    System.err.println("[Error:ServerTCP] Could not listen on port: " + ServerTCP.tcpPort);
	    listening = false;
	}

	starting = false;

	if (listening) {
	    System.out.println("[INFO:ServerTCP] ServerTCP listening on port: " + ServerTCP.tcpPort);
	    while (listening) {
		try {
		    // blocking until new client connect
		    new ServerTCPConnThread(this, serverSocket.accept()).start();
		} catch (IOException e) {
		    System.err.println("[Error:ServerTCP] Could not start new thread.");
		}
	    }

	    try {
		serverSocket.close();
	    } catch (IOException e) {
		System.err.println("[Error:ServerTCP] Could not close port.");
	    }
	} // listening
	System.out.println("[INFO:ServerTCP] ServerTCP stopped");
    }

    /**
     * Set new TCP-Port.
     *
     * @param newTcpPort
     *            new Port number to be set
     */
    public void setPort(final int newTcpPort) {
	ServerTCP.tcpPort = newTcpPort;
    }

    /**
     * Get current TCP-Port number.
     *
     * @return TCP-Port number
     */
    public int getPort() {
	return ServerTCP.tcpPort;
    }

    /**
     * Return Status of Server startup.
     *
     * @return true if Server is still starting
     */
    public boolean isServerStarting() {
	return starting;
    }

    /**
     * Return status of Server-Socket is Listening.
     *
     * @return true if socket is open
     */
    public boolean isServerListening() {
	return listening;
    }

    /**
     * Disconnects all clients and stop Server-Socket.
     */
    public void stopServer() {
	listening = false;
    }

    /**
     * Adds a new Client to the connected Client list.
     *
     * @param newClient
     *            new ConnectionThread Client
     */
    public void addListeningClient(final ServerTCPConnThread newClient) {
	this.clientList.add(newClient);
    }

    /**
     * Removes a Client from the connected Client list.
     *
     * @param client
     *            ConnectionThread Client to be removed
     */
    public void removeListeningClient(final ServerTCPConnThread client) {
	clientList.remove(client);
    }

    /**
     * Returns the number of connected Clients.
     *
     * @return number of connected Clients
     */
    public int numberOfConnectedClients() {
	return this.clientList.size();
    }

    /**
     * Print the currently connected Clients.
     */
    public void printConnectedClients() {
	ServerTCPConnThread client;

	//System.out.println("[INFO:ServerTCP] Clients online: " + this.clientList.size());
	final Iterator<ServerTCPConnThread> iterator = this.clientList.iterator();
	while (iterator.hasNext()) {
	    client = iterator.next();
	    System.out.println(" --> " + client.getName());
	}
    }

}
